#!/usr/bin/env python3
# Rainbow Invaders by Mike Cook

import RPi.GPIO as io
import pygame
import time, random
from apa102bang import Apa102bang

def main():
  global beam,gameOn,bombHeight,bombCol
  print("Rainbow Invaders - Ctr C to quit")
  init()
  gameOn = True ; yourScore = 0
  bombHit = False ; bombHeight = 8
  hits = 3 ; sky.setLed(0,base[hits])
  sky.show() ; time.sleep(gameSpeed)
  nextBombMove = time.time()+gameSpeed
  bombCol = (0,0,0) ; setBombCol()
  while 1:
    while gameOn:
      if time.time()>nextBombMove:
         bombHeight -=1
         showBomb(bombHeight)
         nextBombMove = time.time()+gameSpeed
      yourScore = fireBeam(bombHeight,yourScore)
      if bombHeight == 0 :
         sky.setAll((0,0,0))
         sky.setLed(0,(255,0,0))
         sky.show() # explosion
         gameSound[3].play() # base hit
         time.sleep(2.5) # time to play
         hits -= 1
         if hits == 0:
           gameOn = False
         else:
           print("Base hit:-",hits,"from destruction")
           sky.setLed(0,base[hits])
           setBombCol()
           bombHeight = 8
    sky.show()
    print("Base destroyed:- your score",yourScore)
    time.sleep(gameSpeed * 10)
    print("Push any button for a new game")
    while io.input(fireR) and io.input(fireG) and io.input(fireB):
       pass
    print("New game starting")
    hits = 3
    gameOn = True
    bombHeight = 8
    sky.setLed(0,base[hits])
    sky.show()
    setBombCol()
    while (not io.input(fireR)) | (not io.input(fireG)) | (not io.input(fireB)):
        pass

def getBeam():
    m = 128
    b = ((not io.input(fireR))*m,(not io.input(fireG))*m,(not io. input(fireB))*m)
    return b

def showBeam(col,limit):
    for i in range(1,limit):
       sky.setLed(i,col)
       sky.show()
       time.sleep(0.05)

def showBomb(pos):
    global bombCol
    showBeam((0,0,0),8)
    sky.setLed(pos,bombCol)
    sky.show()

def fireBeam(far,score):
    global beam,bombCol,bombHeight
    if getBeam() != beam: # laser fire
      beam = getBeam()
      if beam != (0,0,0):
           gameSound[1].play()
      showBeam(beam,far)
      if beam == bombCol : #hit OK
        time.sleep(0.2)
        gameSound[2].play() #hit sound
        score += far
        time.sleep(1.5) # allow hit to finish
        bombHeight = 8
        setBombCol()
    return score

def setBombCol():
   global bombCol
   gameSound[0].play() # bomb incoming
   while 1 :
     r = random.randint(0,1) * 128
     g = random.randint(0,1) * 128
     b = random.randint(0,1) * 128
     if not(r==0 and g==0 and b==0) and bombCol != (r,g,b) : break
   bombCol = (r,g,b)

def init():
   global fireR,fireG,fireB,beam,sky,gameSpeed,base,gameSound
   random.seed()
   io.setmode(io.BCM)
   fireR = 4 ; fireG = 3 ; fireB = 2
   io.setup([fireR,fireG,fireB],io.IN, pull_up_down=io.PUD_UP)
   beam = getBeam()
   length = 8 ; brightness = 4
   sky = Apa102bang(length,23,24,brightness)
   gameSpeed = 0.3 # the step speed of game in seconds
   pygame.mixer.quit()
   pygame.mixer.init(frequency=22050, size=-16, channels=2, buffer=512)
   base = [(0,0,0),(32,0,32),(0,64,0),(0,128,128)] # base colors
   soundEffects = ["ComputerBeeps2","Laser1","Pop","Screech"]
   gameSound = [pygame.mixer.Sound("sounds/"+soundEffects[sound] + ".wav") for sound in range(0,4)]

# Main program logic:
if __name__ == "__main__":
  try:
    main()
  except KeyboardInterrupt:
    pass
# turn off all LEDs
sky.setAll((0,0,0))
sky.show()