#!/usr/bin/env python3
# -*- coding: utf8 -*-
# RFID reader class RC522 through SPI

import RPi.GPIO as io
io.setwarnings(False)
import spi
import signal
import time
  
class Rc522rfid:
  MAX_LEN = 16
  OK       = 0
  NOTAGERRERR = 1
  ERR      = 2
  keySectors = [0,3,7,11,15,19,23,27,31,35,39,43,47,51,55,59,63]

  PICC_REQIDL = 0x26
  PICC_AUTHENT1A = 0x60
  
  serNum = []
  
  def __init__(self):
    global reader  
    reader = spi.openSPI(device="/dev/spidev0.0",
                            mode=0,
                            speed=1000000)
    io.setmode(io.BCM)
    io.setup(22, io.OUT)
    io.output(22, 1)
    self.RC522_Init()
    
  def RC522_Init(self): # initilise
    io.output(22, 1)  
    self.RC522_Reset();       
    self.Write_RC522(0x2A, 0x8D)
    self.Write_RC522(0x2B, 0x3E)
    self.Write_RC522(0x2D, 30)
    self.Write_RC522(0x2C, 0)   
    self.Write_RC522(0x15, 0x40)
    self.Write_RC522(0x11, 0x3D)
    self.AntennaOn()
  
  def RC522_Reset(self):
    self.Write_RC522(0x01, 0x0F)
  
  def Write_RC522(self, addr, val):
    spi.transfer(reader,((addr<<1) & 0x7E, val))
  
  def Read_RC522(self, addr):
    val = (0x0, 0x0)  
    val = spi.transfer(reader, (((addr<<1)&0x7E) | 0x80, 0))
    return val[1]
  
  def SetBitMask(self, reg, mask):
    tmp = self.Read_RC522(reg)
    self.Write_RC522(reg, tmp | mask)
    
  def ClearBitMask(self, reg, mask):
    tmp = self.Read_RC522(reg);
    self.Write_RC522(reg, tmp & (~mask))
  
  def AntennaOn(self):
    temp = self.Read_RC522(0x14)
    if(~(temp & 0x03)):
      self.SetBitMask(0x14, 0x03)
  
  def AntennaOff(self):
    self.ClearBitMask(0x14, 0x03)
        
  def ToCard(self,command,sendData):
    backData = []
    backLen = 0
    status = self.ERR
    irqEn = 0x00
    waitIRq = 0x00
    lastBits = None
    n = 0 ; i = 0    
    if command == 0x0E:
      irqEn = 0x12
      waitIRq = 0x10
    if command == 0x0C:
      irqEn = 0x77
      waitIRq = 0x30    
    self.Write_RC522(0x02, irqEn|0x80)
    self.ClearBitMask(0x04, 0x80)
    self.SetBitMask(0x0A, 0x80)    
    self.Write_RC522(0x01, 0x00);      
    while(i<len(sendData)):
      self.Write_RC522(0x09, sendData[i])
      i = i+1    
    self.Write_RC522(0x01, command)      
    if command == 0x0C:
      self.SetBitMask(0x0D, 0x80)   
    i = 2000
    while True:
      n = self.Read_RC522(0x04)
      i = i - 1
      if ~((i!=0) and ~(n&0x01) and ~(n&waitIRq)):  
        break    
    self.ClearBitMask(0x0D, 0x80)  
    if i != 0:
      if (self.Read_RC522(0x06) & 0x1B)==0x00:
        status = self.OK
        if n & irqEn & 0x01:
          status = self.NOTAGERRERR      
        if command == 0x0C:
          n = self.Read_RC522(0x0A)
          lastBits = self.Read_RC522(0x0C) & 0x07
          if lastBits != 0:
            backLen = (n-1)*8 + lastBits
          else:
            backLen = n*8          
          if n == 0:
            n = 1
          if n > self.MAX_LEN:
            n = self.MAX_LEN   
          i = 0
          while i<n:
            backData.append(self.Read_RC522(0x09))
            i = i + 1;
      else:
        status = self.ERR
    return (status,backData,backLen) 
  
  def Anticoll(self):
    backData = []
    serNumCheck = 0   
    serNum = []  
    self.Write_RC522(0x0D, 0x00)    
    serNum.append(0x93)
    serNum.append(0x20)    
    (status,backData,backBits) = self.ToCard(0x0C,serNum)   
    if(status == self.OK):
      i=0
      if len(backData)==5:
        while i<4:
          serNumCheck = serNumCheck ^ backData[i]
          i = i + 1
        if serNumCheck != backData[i]:
          status = self.ERR
      else:
        status = self.ERR  
    return (status,backData)
  
  def CalulateCRC(self, pIndata):
    self.ClearBitMask(0x05, 0x04)
    self.SetBitMask(0x0A, 0x80);
    i = 0
    while i<len(pIndata):
      self.Write_RC522(0x09, pIndata[i])
      i = i + 1
    self.Write_RC522(0x01, 0x03)
    i = 0xFF
    while True:
      n = self.Read_RC522(0x05)
      i = i - 1
      if not ((i != 0) and not (n & 0x04)):
        break
    pOutData = []
    pOutData.append(self.Read_RC522(0x22))
    pOutData.append(self.Read_RC522(0x21))
    return pOutData
 
  def SelectTag(self, serNum):
    backData = []
    buf = []
    buf.append(0x93)
    buf.append(0x70)
    i = 0
    while i<5:
      buf.append(serNum[i])
      i = i + 1
    pOut = self.CalulateCRC(buf)
    buf.append(pOut[0])
    buf.append(pOut[1])
    (status, backData, backLen) = self.ToCard(0x0C, buf)    
    if (status == self.OK) and (backLen == 0x18):
      return    backData[0]
    else:
      return 0
  
  def Auth(self, authMode, BlockAddr, Sectorkey, serNum):
    buff = []
    buff.append(authMode)
    buff.append(BlockAddr)
    i = 0
    while(i < len(Sectorkey)):
      buff.append(Sectorkey[i])
      i = i + 1
    i = 0
    while(i < 4):
      buff.append(serNum[i])
      i = i +1
    (status, backData, backLen) = self.ToCard(0x0E,buff)
    return status
  
  def StopCrypto1(self):
    self.ClearBitMask(0x08, 0x08)

  def Read(self, blockAddr):
    recvData = []
    recvData.append(0x30)
    recvData.append(blockAddr)
    pOut = self.CalulateCRC(recvData)
    recvData.append(pOut[0])
    recvData.append(pOut[1])
    (status, backData, backLen) = self.ToCard(0x0C, recvData)
    if not(status == self.OK):
      print ("Error while reading!")
    if len(backData) == 16:
      return backData
    else:
      return -1
    
  def RC522_Request(self, reqMode):
    status = None ; backBits = None
    TagType = []    
    self.Write_RC522(0x0D, 0x07)   
    TagType.append(reqMode);
    (status,backData,backBits) = self.ToCard(0x0C, TagType)  
    if ((status != self.OK) | (backBits != 0x10)):
      status = self.ERR      
    return (status,backBits)
  
  def RC522_Write(self, blockAddr, writeData):
    buff = []
    buff.append(0xA0)
    buff.append(blockAddr)
    crc = self.CalulateCRC(buff)
    buff.append(crc[0])
    buff.append(crc[1])
    (status, backData, backLen) = self.ToCard(0x0C, buff)
    if not(status == self.OK) or not(backLen == 4) or not((backData[0] & 0x0F) == 0x0A):
        status = self.ERR
    if status == self.OK:
        buf = []
        for i in range(0,16):
            buf.append(writeData[i])
        crc = self.CalulateCRC(buf)
        buf.append(crc[0])
        buf.append(crc[1])
        (status, backData, backLen) = self.ToCard(0x0C,buf)
        if not(status == self.OK) or not(backLen == 4) or not((backData[0] & 0x0F) == 0x0A):
            print ("Error while writing")

  def RC522_WaitForCardRemoved(self): # blocks until card removed
     cardRemoved = False
     while not(cardRemoved):
        (status1,TagType) = self.RC522_Request(self.PICC_REQIDL)
        (status,TagType) = self.RC522_Request(self.PICC_REQIDL)
        if status1 != self.OK and status != self.OK:
           cardRemoved = True
    
   
  def RC522_ReadCard(self):
     (status,TagType) = self.RC522_Request(self.PICC_REQIDL)
     if status == self.OK:
         (status,uid) = self.Anticoll() # get the card's UID
         if len(uid) == 5:
            token  = (uid[0] << 24) | (uid[1] << 16) | (uid[2] << 8) | uid[3]
            return token
         else:
            return -1 # corrupt read
     else:  
         return -1 # no card found
      
  def RC522_GetCard(self): # blocks until next card presented
     self.RC522_WaitForCardRemoved()
     cardNumber = -1
     while cardNumber == -1:
       cardNumber = self.RC522_ReadCard()
     return cardNumber

  def RC522_GetSector(self,key,sector):
     # get card to read from
     (status,TagType) = self.RC522_Request(self.PICC_REQIDL)
     while status != self.OK: # looking for a card
        (status,TagType) = self.RC522_Request(self.PICC_REQIDL) 
     (status,uid) = self.Anticoll()
     if status == self.OK:
         self.SelectTag(uid)
         status = self.Auth(self.PICC_AUTHENT1A, sector, key, uid)
         if status == self.OK:
            block = self.Read(sector)
            self.StopCrypto1()
         else:
            print("Authencation error")
            return -1
     else :
         print("error in status")
         return -1
     return block
    
  def RC522_WriteSector(self,key,sector,block):
    if sector in self.keySectors :
      print("It is not permitted to write to a Key sector")
      return
    # get card to write to
    (status,TagType) = self.RC522_Request(self.PICC_REQIDL)
    while status != self.OK: # looking for a card
       (status,TagType) = self.RC522_Request(self.PICC_REQIDL) 
    (status,uid) = self.Anticoll()
    if status == self.OK:
       self.SelectTag(uid)
       status = self.Auth(self.PICC_AUTHENT1A, sector, key, uid)
       if status == self.OK:
          self.RC522_Write(sector, block)
          self.StopCrypto1()
       else:
          print("Authencation error")
    else :
       print("error in status")

  
